@@ -98,14 +98,16 @@ Standard errors are not available unless you fix it.")
     }
 
     if (length(fixed.param) > 0){
-      args <- c(list(data = data, coord = coord, cov.mod = cov.mod, marge = "emp"), fixed.param)
+      args <- c(list(data = data, coord = coord, cov.mod = paste("i", cov.mod, sep=""),
+                     marge = "emp"), fixed.param)
       cov.start <- do.call("fitcovariance", args)$param
     }
 
     else
-      cov.start <- fitcovariance(data, coord, cov.mod, marge = "emp")$param
+      cov.start <- fitcovariance(data, coord, paste("i", cov.mod, sep=""),
+                                 marge = "emp")$param
 
-    start <- c(list(alpha = 0.5), as.list(cov.start), start)
+    start <- c(as.list(cov.start), start)
     start <- start[!(param %in% names(list(...)))]
   }
   
@@ -124,6 +126,14 @@ Standard errors are not available unless you fix it.")
   
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
+
+  ##We use the parscale option to help the optimizer
+  ##We do not overwrite user config
+  if (is.null(control$parscale)){
+    parscale <- abs(unlist(start))
+    parscale[parscale == 0] <- 1
+    control$parscale <- parscale
+  }
   
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
@@ -269,7 +279,8 @@ Standard errors are not available unless you fix it.")
                          smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
   
   ext.coeff <- function(h)
-    1 + sqrt(1 - 1/2 * (cov.fun(h) + 1))
+    2 * param["alpha"] + (1 - param["alpha"]) *
+      (1 + sqrt(1 - 1/2 * (cov.fun(h) + 1)))
 
   fitted <- list(fitted.values = opt$par, std.err = std.err, std.err.type = std.err.type,
                  var.cov = var.cov, param = param, cov.fun = cov.fun, fixed = unlist(fixed.param),
@@ -394,8 +405,8 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
 
   if (missing(start)) {
 
-    start <- .start.schlatherind(data, coord, cov.mod, loc.model, scale.model,
-                                 shape.model, method = method, ...)
+    start <- .start.schlatherind(data, coord, covariables, cov.mod, loc.form,
+                                 scale.form, shape.form, method = method, ...)
 
     start <- start[!(param %in% names(list(...)))]
   
@@ -415,7 +426,15 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
-  
+
+  ##We use the parscale option to help the optimizer
+  ##We do not overwrite user config
+  if (is.null(control$parscale)){
+    parscale <- abs(unlist(start))
+    parscale[parscale == 0] <- 1
+    control$parscale <- parscale
+  }
+    
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
 
@@ -579,7 +598,8 @@ Standard errors are not available unless you fix it.")
                         smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
   
   ext.coeff <- function(h)
-    1 + param["alpha"] + (1 - param["alpha"]) * sqrt(1 - 1/2 * (cov.fun(h) + 1))
+    2 * param["alpha"] + (1 - param["alpha"]) *
+      (1 + sqrt(1 - 1/2 * (cov.fun(h) + 1)))
   
   fitted <- list(fitted.values = opt$par, std.err = std.err, std.err.type = std.err.type,
                  var.cov = var.cov, fixed = unlist(fixed.param), param = param,
