@@ -17,41 +17,41 @@ void smithfull(double *data, double *distVec, int *nSite,
   mahalDist = (double *)R_alloc(nPairs, sizeof(double));
   frech = (double *)R_alloc(*nSite * *nObs, sizeof(double));
   
-  *dns = 1.0;
   //Some preliminary steps: Valid points?
   if (*fitmarge){
     for (i=0;i<*nSite;i++){
-      if (scales[i] <= 0){
-	//printf("scales <= 0!!!\n");
-	*dns += R_pow_di(1 - scales[i], 2);
-	scales[i] = .1;
-      }
-      
-      if (shapes[i] <= -1){
-	//printf("shapes <= -1!!!\n");
-	*dns += R_pow_di(shapes[i], 2);
-	shapes[i] = 0.0;
+      if ((scales[i] <= 0) | (shapes[i] <= -1)){
+	*dns = MINF;
+	return;
       }
     }
   }
   
   //Stage 1: Computing the Mahalanobis distance
-  *dns += mahalDistFct(distVec, nPairs, cov11,
-		       cov12, cov22, mahalDist);
+  *dns = mahalDistFct(distVec, nPairs, cov11,
+		      cov12, cov22, mahalDist);
+
+  if (*dns != 0.0)
+    return;
   
   //Stage 2: Transformation to unit Frechet
-  if (*fitmarge)
-    *dns += gev2frech(data, *nObs, *nSite, locs, scales,
-		      shapes, jac, frech);
-  else {
-    for (i=0;i<(*nSite * *nObs);i++){
-      frech[i] = data[i];
-      jac[i] = 0.0;    
-    }
+  if (*fitmarge){
+    *dns = gev2frech(data, *nObs, *nSite, locs, scales,
+		     shapes, jac, frech);
+
+    if (*dns != 0.0)
+      return;
+
+    *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
   }
   
-  *dns *= lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
+  else {
+    for (i=0;i<(*nSite * *nObs);i++)
+      jac[i] = 0.0;    
 
+    *dns = lpliksmith(data, mahalDist, jac, *nObs, *nSite);
+  }
+  
   if (!R_FINITE(*dns))
     *dns = MINF;
 
@@ -79,22 +79,30 @@ void smithdsgnmat(double *data, double *distVec, int *nSite, int *nObs,
   shapes = (double *)R_alloc(*nSite, sizeof(double));
   frech = (double *)R_alloc(*nSite * *nObs, sizeof(double));
   
-  *dns = 1.0;
   //Stage 1: Computing the Mahalanobis distance
-  *dns += mahalDistFct(distVec, nPairs, cov11, cov12,
-		       cov22, mahalDist);
+  *dns = mahalDistFct(distVec, nPairs, cov11, cov12,
+		      cov22, mahalDist);
+
+  if (*dns != 0.0)
+    return;
 
   //Stage 2: Computing the GEV parameters using the design matrix
-  *dns += dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
-			loccoeff, scalecoeff, shapecoeff, *nSite,
-			*nloccoeff, *nscalecoeff, *nshapecoeff,
-			locs, scales, shapes);
+  *dns = dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
+		       loccoeff, scalecoeff, shapecoeff, *nSite,
+		       *nloccoeff, *nscalecoeff, *nshapecoeff,
+		       locs, scales, shapes);
+
+  if (*dns != 0.0)
+    return;
 
   //Stage 3: Transformation to unit Frechet
-  *dns += gev2frech(data, *nObs, *nSite, locs, scales, shapes,
-		    jac, frech);
+  *dns = gev2frech(data, *nObs, *nSite, locs, scales, shapes,
+		   jac, frech);
+
+  if (*dns != 0.0)
+    return;
   
-  *dns *= lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
+  *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
     
   //Stage 5: Removing the penalizing terms (if any)
   // 1- For the location parameter
